import 'package:dio/dio.dart';
import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';

mixin BaseViewState{

  /// 防止页面销毁后,异步任务才完成,导致报错
  bool disposed = false;

  /// 当前的页面状态,默认为busy,可在viewModel的构造方法中指定;
  ViewState viewState = ViewState.busy;

  ViewStateErrorType errorType;

  EasyRefreshController _refreshController = EasyRefreshController();

  set refreshController(EasyRefreshController easyRefreshController){
    this._refreshController = easyRefreshController;
  }



  EasyRefreshController get refreshController => _refreshController;

  bool isInit = false;

  void getOtherData(){}

  ///获取请求的数据，一般用于简单的页面请求
  RequestInfo getRequestInfo();

  void beforeRequest(){}


  ///---------------------------------------------页面状态------------------------------------------------------

  ViewStateError viewStateError;

  bool get isBusy => viewState == ViewState.busy;

  bool get isIdle => viewState == ViewState.idle;

  bool get isError => viewState == ViewState.error;


}





/// 页面状态类型
enum ViewState {
  idle,
  busy,
  error
}

/// 错误类型
enum ViewStateErrorType {
  defaultError,  //默认报错
  networkTimeOutError, //超时错误
  unauthorizedError, //为授权(一般为未登录)
  noDataError, //无数据
  httpError,  //请求错误
  notSuccessError,  //请求不成功错误
  jsonParsingError, //json解析错误
}


class ViewStateError {
  ViewStateErrorType _errorType;
  String errorMessage;
  dynamic error;

  ViewStateError(this._errorType, {this.errorMessage, this.error}) {
    _errorType ??= ViewStateErrorType.defaultError;
  }

  ViewStateErrorType get errorType => _errorType;

  /// 以下变量是为了代码书写方便,加入的get方法.严格意义上讲,并不严谨
  get isDefaultError => _errorType == ViewStateErrorType.defaultError;
  get isNetworkTimeOut => _errorType == ViewStateErrorType.networkTimeOutError;
  get isUnauthorized => _errorType == ViewStateErrorType.unauthorizedError;
  get isNoDataError => _errorType == ViewStateErrorType.unauthorizedError;
  get isNotSuccessError => _errorType == ViewStateErrorType.notSuccessError;
  get isHttpError => _errorType == ViewStateErrorType.httpError;


  @override
  String toString() {
    return 'ViewStateError{errorType: $_errorType, errorMessage: $errorMessage, error:$error}';
  }
}